@@ -1,33 +1,31 @@
-""" 
+"""
  @file
  @brief This file contains the blender file listview, used by the 3d animated titles screen
  @author Jonathan Thomas <jonathan@openshot.org>
- 
+
  @section LICENSE
- 
+
  Copyright (c) 2008-2018 OpenShot Studios, LLC
  (http://www.openshotstudios.com). This file is part of
  OpenShot Video Editor (http://www.openshot.org), an open-source project
  dedicated to delivering high quality video editing and animation solutions
  to the world.
- 
+
  OpenShot Video Editor is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
- 
+
  OpenShot Video Editor is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public License
  along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
  """
 
-import codecs
 import os
-import uuid
 import shutil
 import subprocess
 import sys
@@ -46,20 +44,7 @@ from classes.query import File
 from classes.app import get_app
 from windows.models.blender_model import BlenderModel
 
-try:
-    import json
-except ImportError:
-    import simplejson as json
-
-
-class QBlenderEvent(QEvent):
-    """ A custom Blender QEvent, which can safely be sent from the Blender thread to the Qt thread (to communicate) """
-
-    def __init__(self, id, data=None, *args):
-        # Invoke parent init
-        QEvent.__init__(self, id)
-        self.data = data
-        self.id = id
+import json
 
 
 class BlenderListView(QListView):
@@ -133,8 +118,8 @@ class BlenderListView(QListView):
                 self.params[param["name"]] = _(param["default"])
 
                 # create spinner
-                widget = QTextEdit()
-                widget.setText(_(param["default"]).replace("\\n", "\n"))
+                widget = QPlainTextEdit()
+                widget.setPlainText(_(param["default"]).replace("\\n", "\n"))
                 widget.textChanged.connect(functools.partial(self.text_value_changed, widget, param))
 
             elif param["type"] == "dropdown":
@@ -150,15 +135,20 @@ class BlenderListView(QListView):
                     # override files dropdown
                     param["values"] = {}
                     for file in File.filter():
-                        if file.data["media_type"] in ("image", "video"):
-                            (dirName, fileName) = os.path.split(file.data["path"])
-                            (fileBaseName, fileExtension) = os.path.splitext(fileName)
+                        if file.data["media_type"] not in ("image", "video"):
+                            continue
 
-                            if fileExtension.lower() not in (".svg"):
-                                param["values"][fileName] = "|".join((file.data["path"], str(file.data["height"]),
-                                                                      str(file.data["width"]), file.data["media_type"],
-                                                                      str(file.data["fps"]["num"] / file.data["fps"][
-                                                                          "den"])))
+                        fileName = os.path.basename(file.data["path"])
+                        fileExtension = os.path.splitext(fileName)[1]
+
+                        if fileExtension.lower() in (".svg"):
+                            continue
+
+                        param["values"][fileName] = "|".join(
+                            file.data["path"], str(file.data["height"]),
+                            str(file.data["width"]), file.data["media_type"],
+                            str(file.data["fps"]["num"] / file.data["fps"]["den"])
+                        )
 
                 # Add normal values
                 box_index = 0
@@ -178,7 +168,10 @@ class BlenderListView(QListView):
             elif param["type"] == "color":
                 # add value to dictionary
                 color = QColor(param["default"])
-                self.params[param["name"]] = [color.redF(), color.greenF(), color.blueF()]
+                if "diffuse_color" in param.get("name"):
+                    self.params[param["name"]] = [color.redF(), color.greenF(), color.blueF(), color.alphaF()]
+                else:
+                    self.params[param["name"]] = [color.redF(), color.greenF(), color.blueF()]
 
                 widget = QPushButton()
                 widget.setText("")
@@ -198,51 +191,49 @@ class BlenderListView(QListView):
         self.init_slider_values()
 
     def spinner_value_changed(self, param, value):
-        log.info('Animation param being changed: %s' % param["name"])
         self.params[param["name"]] = value
-        log.info('New value of param: %s' % value)
+        log.info('Animation param %s set to %s' % (param["name"], value))
 
     def text_value_changed(self, widget, param, value=None):
         try:
-            # Attempt to load value from QTextEdit (i.e. multi-line)
+            # Attempt to load value from QPlainTextEdit (i.e. multi-line)
             if not value:
                 value = widget.toPlainText()
-        except:
-            pass
-        log.info('Animation param being changed: %s' % param["name"])
+        except Exception:
+            return
         self.params[param["name"]] = value.replace("\n", "\\n")
-        log.info('New value of param: %s' % value)
+        # XXX: This will log every individual KEYPRESS in the text field.
+        # log.info('Animation param %s set to %s' % (param["name"], value))
 
     def dropdown_index_changed(self, widget, param, index):
-        log.info('Animation param being changed: %s' % param["name"])
         value = widget.itemData(index)
         self.params[param["name"]] = value
-        log.info('New value of param: %s' % value)
+        log.info('Animation param %s set to %s' % (param["name"], value))
 
     def color_button_clicked(self, widget, param, index):
         # Get translation object
         _ = get_app()._tr
 
         # Show color dialog
-        log.info('Animation param being changed: %s' % param["name"])
         color_value = self.params[param["name"]]
-        log.info('Value of param: %s' % color_value)
         currentColor = QColor("#FFFFFF")
         if len(color_value) == 3:
-            #currentColor = QColor(color_value[0], color_value[1], color_value[2])
             currentColor.setRgbF(color_value[0], color_value[1], color_value[2])
         newColor = QColorDialog.getColor(currentColor, self, _("Select a Color"),
                                          QColorDialog.DontUseNativeDialog)
         if newColor.isValid():
             widget.setStyleSheet("background-color: {}".format(newColor.name()))
-            self.params[param["name"]] = [newColor.redF(), newColor.greenF(), newColor.blueF()]
-            log.info('New value of param: %s' % newColor.name())
+            if "diffuse_color" in param.get("name"):
+                self.params[param["name"]] = [newColor.redF(), newColor.greenF(), newColor.blueF(), newColor.alphaF()]
+            else:
+                self.params[param["name"]] = [newColor.redF(), newColor.greenF(), newColor.blueF()]
+            log.info('Animation param %s set to %s' % (param["name"], newColor.name()))
 
     def generateUniqueFolder(self):
         """ Generate a new, unique folder name to contain Blender frames """
 
         # Assign a new unique id for each template selected
-        self.unique_folder_name = str(uuid.uuid1())
+        self.unique_folder_name = str(self.app.project.generate_id())
 
         # Create a folder (if it does not exist)
         if not os.path.exists(os.path.join(info.BLENDER_PATH, self.unique_folder_name)):
@@ -250,26 +241,32 @@ class BlenderListView(QListView):
 
     def disable_interface(self, cursor=True):
         """ Disable all controls on interface """
+
+        # Store keyboard-focused widget
+        self.focus_owner = self.win.focusWidget()
+
         self.win.btnRefresh.setEnabled(False)
         self.win.sliderPreview.setEnabled(False)
-        self.win.buttonBox.setEnabled(False)
+        self.win.btnRender.setEnabled(False)
 
         # Show 'Wait' cursor
         if cursor:
             QApplication.setOverrideCursor(Qt.WaitCursor)
 
+    @pyqtSlot()
     def enable_interface(self):
         """ Disable all controls on interface """
         self.win.btnRefresh.setEnabled(True)
         self.win.sliderPreview.setEnabled(True)
-        self.win.buttonBox.setEnabled(True)
+        self.win.btnRender.setEnabled(True)
 
-        # Restore normal cursor
+        # Restore normal cursor and keyboard focus
         QApplication.restoreOverrideCursor()
+        if self.focus_owner:
+            self.focus_owner.setFocus()
 
     def init_slider_values(self):
         """ Init the slider and preview frame label to the currently selected animation """
-        log.info("init_slider_values")
 
         # Get current preview slider frame
         preview_frame_number = self.win.sliderPreview.value()
@@ -298,16 +295,15 @@ class BlenderListView(QListView):
     def btnRefresh_clicked(self, checked):
 
         # Render current frame
-        log.info("btnRefresh_clicked")
         preview_frame_number = self.win.sliderPreview.value()
-        self.Render(preview_frame_number)
+        self.preview_timer.start()
 
+    @pyqtSlot()
     def render_finished(self):
-        log.info("RENDER FINISHED!")
 
         # Add file to project
         final_path = os.path.join(info.BLENDER_PATH, self.unique_folder_name, self.params["file_name"] + "%04d.png")
-        log.info('Adding to project files: %s' % final_path)
+        log.info('RENDER FINISHED! Adding to project files: %s' % final_path)
 
         # Add to project files
         self.win.add_file(final_path)
@@ -315,13 +311,8 @@ class BlenderListView(QListView):
         # Enable the Render button again
         self.win.close()
 
-    def close_window(self):
-        log.info("CLOSING WINDOW")
-
-        # Close window
-        self.close()
-
-    def update_progress_bar(self, current_frame, current_part, max_parts):
+    @pyqtSlot(int)
+    def update_progress_bar(self, current_frame):
 
         # update label and preview slider
         self.win.sliderPreview.setValue(current_frame)
@@ -331,7 +322,6 @@ class BlenderListView(QListView):
 
     def sliderPreview_valueChanged(self, new_value):
         """Get new value of preview slider, and start timer to Render frame"""
-        log.info('sliderPreview_valueChanged: %s' % new_value)
         if self.win.sliderPreview.isEnabled():
             self.preview_timer.start()
 
@@ -342,11 +332,11 @@ class BlenderListView(QListView):
 
     def preview_timer_onTimeout(self):
         """Timer is ready to Render frame"""
-        log.info('preview_timer_onTimeout')
         self.preview_timer.stop()
 
         # Update preview label
         preview_frame_number = self.win.sliderPreview.value()
+        log.info('Previewing frame %s' % preview_frame_number)
 
         # Render current frame
         self.Render(preview_frame_number)
@@ -445,9 +435,9 @@ class BlenderListView(QListView):
         project_params = {}
 
         # Append on some project settings
-        project_params["fps"] = project.get(["fps"])
-        project_params["resolution_x"] = project.get(["width"])
-        project_params["resolution_y"] = project.get(["height"])
+        project_params["fps"] = project.get("fps")
+        project_params["resolution_x"] = project.get("width")
+        project_params["resolution_y"] = project.get("height")
 
         if is_preview:
             project_params["resolution_percentage"] = 50
@@ -455,14 +445,8 @@ class BlenderListView(QListView):
             project_params["resolution_percentage"] = 100
         project_params["quality"] = 100
         project_params["file_format"] = "PNG"
-        if is_preview:
-            # preview mode - use offwhite background (i.e. horizon color)
-            project_params["color_mode"] = "RGB"
-            project_params["alpha_mode"] = "SKY"
-        else:
-            # render mode - transparent background
-            project_params["color_mode"] = "RGBA"
-            project_params["alpha_mode"] = "TRANSPARENT"
+        project_params["color_mode"] = "RGBA"
+        project_params["alpha_mode"] = 1
         project_params["horizon_color"] = (0.57, 0.57, 0.57)
         project_params["animation"] = True
         project_params["output_path"] = os.path.join(info.BLENDER_PATH, self.unique_folder_name,
@@ -479,17 +463,16 @@ class BlenderListView(QListView):
         version_message = ""
         if version:
             version_message = _("\n\nVersion Detected:\n{}").format(version)
+            log.error("Blender version detected: {}".format(version))
 
         if command_output:
             version_message = _("\n\nError Output:\n{}").format(command_output)
+            log.error("Blender error output:\n{}".format(command_output))
 
-        # show error message
-        blender_version = "2.78"
-        # Handle exception
         msg = QMessageBox()
         msg.setText(_(
             "Blender, the free open source 3D content creation suite is required for this action (http://www.blender.org).\n\nPlease check the preferences in OpenShot and be sure the Blender executable is correct.  This setting should be the path of the 'blender' executable on your computer.  Also, please be sure that it is pointing to Blender version {} or greater.\n\nBlender Path:\n{}{}").format(
-            blender_version, s.get("blender_command"), version_message))
+            info.BLENDER_MIN_VERSION, s.get("blender_command"), version_message))
         msg.exec_()
 
         # Enable the Render button again
@@ -520,30 +503,52 @@ class BlenderListView(QListView):
 
         # Force the Frame to 1 frame (for previewing)
         if frame:
-            user_params += "\n\n#ONLY RENDER 1 FRAME FOR PREVIEW\n"
+            user_params += "\n#ONLY RENDER 1 FRAME FOR PREVIEW\n"
             user_params += "params['{}'] = {}\n".format("start_frame", frame)
             user_params += "params['{}'] = {}\n".format("end_frame", frame)
-            user_params += "\n\n#END ONLY RENDER 1 FRAME FOR PREVIEW\n"
+            user_params += "#END ONLY RENDER 1 FRAME FOR PREVIEW\n"
+
+        # If GPU rendering is selected, see if GPU enable code is available
+        s = settings.get_settings()
+        gpu_code_body = None
+        if s.get("blender_gpu_enabled"):
+            gpu_enable_py = os.path.join(info.PATH, "blender", "scripts", "gpu_enable.py")
+            try:
+                f = open(gpu_enable_py, 'r')
+                gpu_code_body = f.read()
+            except IOError as e:
+                log.error("Could not load GPU enable code! {}".format(e))
+
+        if gpu_code_body:
+            log.info("Injecting GPU enable code from {}".format(gpu_enable_py))
+            user_params += "\n#ENABLE GPU RENDERING\n"
+            user_params += gpu_code_body
+            user_params += "\n#END ENABLE GPU RENDERING\n"
 
         # Open new temp .py file, and inject the user parameters
         with open(path, 'r') as f:
             script_body = f.read()
 
         # modify script variable
-        script_body = script_body.replace("#INJECT_PARAMS_HERE", user_params)
+        script_body = script_body.replace("# INJECT_PARAMS_HERE", user_params)
 
         # Write update script
-        with codecs.open(path, "w", encoding="UTF-8") as f:
+        with open(path, "w", encoding="UTF-8", errors="strict") as f:
             f.write(script_body)
 
+    @pyqtSlot(str)
     def update_image(self, image_path):
 
         # get the pixbuf
         image = QImage(image_path)
-        scaled_image = image.scaledToHeight(self.win.imgPreview.height(), Qt.SmoothTransformation);
+        scaled_image = image.scaledToHeight(self.win.imgPreview.height(), Qt.SmoothTransformation)
         pixmap = QPixmap.fromImage(scaled_image)
         self.win.imgPreview.setPixmap(pixmap)
 
+    def Cancel(self):
+        """Cancel the current render, if any"""
+        QMetaObject.invokeMethod(self.worker, 'Cancel', Qt.DirectConnection)
+
     def Render(self, frame=None):
         """ Render an images sequence of the current template using Blender 2.62+ and the
         Blender Python API. """
@@ -559,7 +564,9 @@ class BlenderListView(QListView):
 
         # Copy the .py script associated with this template to the temp folder.  This will allow
         # OpenShot to inject the user-entered params into the Python script.
-        shutil.copy(source_script, target_script)
+        # XXX: Note that copyfile() is used instead of copy(), as the original
+        #      file may be readonly, and we don't want to duplicate those permissions
+        shutil.copyfile(source_script, target_script)
 
         # Open new temp .py file, and inject the user parameters
         self.inject_params(target_script, frame)
@@ -632,57 +639,34 @@ class BlenderListView(QListView):
         self.worker = Worker()  # no parent!
 
         # Hook up signals to Background Worker
-        self.worker.closed.connect(self.onCloseWindow)
-        self.worker.finished.connect(self.onRenderFinish)
+        self.worker.closed.connect(self.close)
+        self.worker.finished.connect(self.render_finished)
         self.worker.blender_version_error.connect(self.onBlenderVersionError)
         self.worker.blender_error_nodata.connect(self.onBlenderErrorNoData)
-        self.worker.progress.connect(self.onUpdateProgress)
-        self.worker.image_updated.connect(self.onUpdateImage)
+        self.worker.progress.connect(self.update_progress_bar)
+        self.worker.image_updated.connect(self.update_image)
         self.worker.blender_error_with_data.connect(self.onBlenderErrorMessage)
-        self.worker.enable_interface.connect(self.onRenableInterface)
+        self.worker.enable_interface.connect(self.enable_interface)
 
         # Move Worker to new thread, and Start
         self.worker.moveToThread(self.background)
         self.background.start()
 
-    # Signal when to close window (1001)
-    def onCloseWindow(self):
-        log.info('onCloseWindow')
-        self.close()
-
-    # Signal when render is finished (1002)
-    def onRenderFinish(self):
-        log.info('onRenderFinish')
-        self.render_finished()
-
     # Error from blender (with version number) (1003)
+    @pyqtSlot(str)
     def onBlenderVersionError(self, version):
-        log.info('onBlenderVersionError: %s' % version)
         self.error_with_blender(version)
 
     # Error from blender (with no data) (1004)
+    @pyqtSlot()
     def onBlenderErrorNoData(self):
-        log.info('onBlenderErrorNoData')
         self.error_with_blender()
 
-    # Signal when to update progress bar (1005)
-    def onUpdateProgress(self, current_frame, current_part, max_parts):
-        self.update_progress_bar(current_frame, current_part, max_parts)
-
-    # Signal when to update preview image (1006)
-    def onUpdateImage(self, image_path):
-        self.update_image(image_path)
-
     # Signal error from blender (with custom message) (1007)
+    @pyqtSlot(str)
     def onBlenderErrorMessage(self, error):
-        log.info('onBlenderErrorMessage')
         self.error_with_blender(None, error)
 
-    # Signal when to re-enable interface (1008)
-    def onRenableInterface(self):
-        log.info('onRenableInterface')
-        self.enable_interface()
-
 
 class Worker(QObject):
     """ Background Worker Object (to run the Blender commands) """
@@ -691,22 +675,29 @@ class Worker(QObject):
     finished = pyqtSignal()  # 1002
     blender_version_error = pyqtSignal(str)  # 1003
     blender_error_nodata = pyqtSignal()  # 1004
-    progress = pyqtSignal(int, int, int)  # 1005
+    progress = pyqtSignal(int)  # 1005
     image_updated = pyqtSignal(str)  # 1006
     blender_error_with_data = pyqtSignal(str)  # 1007
     enable_interface = pyqtSignal()  # 1008
 
+    @pyqtSlot()
+    def Cancel(self):
+        """Cancel worker render"""
+        self.is_running = False
+        if self.process:
+            # Stop blender process if running
+            self.process.terminate()
+
     @pyqtSlot(str, str, bool)
     def Render(self, blend_file_path, target_script, preview_mode=False):
         """ Worker's Render method which invokes the Blender rendering commands """
-        log.info("QThread Render Method Invoked")
 
         # Init regex expression used to determine blender's render progress
         s = settings.get_settings()
 
         # get the blender executable path
         self.blender_exec_path = s.get("blender_command")
-        self.blender_frame_expression = re.compile(r"Fra:([0-9,]*).*Mem:(.*?) .*Part ([0-9,]*)-([0-9,]*)")
+        self.blender_frame_expression = re.compile(r"Fra:([0-9,]*).*Mem:(.*?) .*Sce:")
         self.blender_saved_expression = re.compile(r"Saved: '(.*.png)(.*)'")
         self.blender_version = re.compile(r"Blender (.*?) ")
         self.blend_file_path = blend_file_path
@@ -728,31 +719,31 @@ class Worker(QObject):
             # Shell the blender command to create the image sequence
             command_get_version = [self.blender_exec_path, '-v']
             command_render = [self.blender_exec_path, '-b', self.blend_file_path, '-P', self.target_script]
+
+            # debug info
+            # NOTE: If the length of the command_render list changes, update to match!
+            log.info("Blender command: {} {} '{}' {} '{}'".format(*command_render))
+
             self.process = subprocess.Popen(command_get_version, stdout=subprocess.PIPE, stderr=subprocess.PIPE, startupinfo=startupinfo)
 
             # Check the version of Blender
             self.version = self.blender_version.findall(str(self.process.stdout.readline()))
 
             if self.version:
-                if float(self.version[0]) < 2.78:
+                if self.version[0] < info.BLENDER_MIN_VERSION:
                     # change cursor to "default" and stop running blender command
                     self.is_running = False
 
-                    # Wrong version of Blender.  Must be 2.62+:
-                    self.blender_version_error.emit(float(self.version[0]))
+                    # Wrong version of Blender.
+                    self.blender_version_error.emit(self.version[0])
                     return
 
-            # debug info
-            log.info(
-                "Blender command: {} {} '{}' {} '{}'".format(command_render[0], command_render[1], command_render[2],
-                                                             command_render[3], command_render[4]))
-
             # Run real command to render Blender project
             self.process = subprocess.Popen(command_render, stdout=subprocess.PIPE, stderr=subprocess.PIPE, startupinfo=startupinfo)
 
         except:
-            # Error running command.  Most likely the blender executable path in the settings
-            # is not correct, or is not the correct version of Blender (i.e. 2.62+)
+            # Error running command.  Most likely the blender executable path in
+            # the settings is incorrect, or is not a supported Blender version
             self.is_running = False
             self.blender_error_nodata.emit()
             return
@@ -770,13 +761,11 @@ class Worker(QObject):
                 self.frame_detected = True
                 current_frame = output_frame[0][0]
                 memory = output_frame[0][1]
-                current_part = output_frame[0][2]
-                max_parts = output_frame[0][3]
 
                 # Update progress bar
                 if not self.preview_mode:
                     # only update progress if in 'render' mode
-                    self.progress.emit(float(current_frame), float(current_part), float(max_parts))
+                    self.progress.emit(int(current_frame))
 
             # Look for progress info in the Blender Output
             output_saved = self.blender_saved_expression.findall(str(line))
@@ -784,15 +773,14 @@ class Worker(QObject):
             # Does it have a match?
             if output_saved:
                 # Yes, we have a match
-                log.info("Image detected from blender regex: %s" % output_saved)
                 self.frame_detected = True
                 image_path = output_saved[0][0]
                 time_saved = output_saved[0][1]
 
                 # Update preview image
+                log.info("Image detected from blender regex: %s" % image_path)
                 self.image_updated.emit(image_path)
 
-
         # Re-enable the interface
         self.enable_interface.emit()
 
@@ -803,13 +791,12 @@ class Worker(QObject):
             self.blender_error_with_data.emit(_("No frame was found in the output from Blender"))
 
         # Done with render (i.e. close window)
-        elif not self.preview_mode:
-            # only close window if in 'render' mode
+        elif self.is_running and not self.preview_mode:
+            # only add file to project data if in 'render' mode and not canceled
             self.finished.emit()
 
         # Thread finished
-        log.info("Blender render thread finished")
-        if self.is_running == False:
+        if not self.is_running:
             # close window if thread was killed
             self.closed.emit()
 
